@@ -24,10 +24,13 @@ import datetime
 import logging
 import os
 import platform
+import sys
 
 import shortuuid  # type: ignore
 import six
 import wandb
+from wandb import env
+from wandb.internal import git_repo
 
 if wandb.TYPE_CHECKING:  # type: ignore
     from typing import (  # noqa: F401 pylint: disable=unused-import
@@ -68,6 +71,7 @@ defaults = dict(
     _problem=Field(str, ("fatal", "warn", "silent",)),
     console="auto",
     _console=Field(str, ("auto", "redirect", "off", "mock", "file", "iowrap",)),
+    git_remote="origin",
 )
 
 # env mapping?
@@ -110,6 +114,45 @@ def _get_python_type():
         return "python"
 
 
+def _is_kaggle():
+    return (
+        os.getenv("KAGGLE_KERNEL_RUN_TYPE") is not None
+        or "kaggle_environments" in sys.modules  # noqa: W503
+    )
+
+
+def _get_program():
+    program = os.getenv(wandb.env.PROGRAM)
+    if program:
+        return program
+
+    try:
+        import __main__  # type: ignore
+
+        program = __main__.__file__
+        if not program:
+            return "<python with no main file>"
+    except (ImportError, AttributeError):
+        return None
+
+    repo = git_repo.GitRepo()
+    root = repo.root
+    if not root:
+        root = os.getcwd()
+    full_path_to_program = os.path.join(
+        root, os.path.relpath(os.getcwd(), root), program
+    )
+    if os.path.exists(full_path_to_program):
+        relative_path = os.path.relpath(full_path_to_program, start=root)
+        if "../" in relative_path:
+            logger.warn("could not save program above cwd: %s" % program)
+            return None
+        return relative_path
+
+    logger.warn("could not find program at %s" % program)
+    return None
+
+
 class CantTouchThis(type):
     def __setattr__(cls, attr, value):
         raise Exception("NO!")
@@ -188,6 +231,9 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
         program=None,
         notebook_name=None,
         disable_code=None,
+        save_code=None,
+        code_program=None,
+        git_remote=None,
         host=None,
         username=None,
         docker=None,
@@ -212,6 +258,8 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
         _early_logger=None,
         _internal_queue_timeout=2,
         _internal_check_process=8,
+        _disable_meta=None,
+        _disable_stats=None,
     ):
         kwargs = locals()
         object.__setattr__(self, "_masked_keys", set(["self", "_frozen"]))
@@ -323,7 +371,7 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
         self.__dict__.update({k: v for k, v in d.items() if v is not None})
         self.__dict__.update({k: v for k, v in kwargs.items() if v is not None})
 
-    def _probe(self):
+    def infer_settings_from_env(self):
         d = {}
         d["jupyter"] = _get_python_type() != "python"
         d["windows"] = platform.system() == "Windows"
@@ -340,8 +388,23 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
             if self.jupyter:
                 console = "off"
             u["console"] = console
+
+        # For code saving, only allow env var override if value from server is true, or
+        # if no preference was specified.
+        if (self.save_code is True or self.save_code is None) and os.getenv(
+            env.SAVE_CODE
+        ) is not None:
+            u["save_code"] = env.should_save_code()
+
+        u["disable_code"] = os.getenv(env.DISABLE_CODE)
         self.update(u)
 
+        # If the settings say to save code, and there's not already a program file,
+        # infer it now.
+        if self.save_code and not self.code_program:
+            code_program = _get_program()
+            self.update(dict(code_program=code_program))
+
     def setdefaults(self, __d=None):
         __d = __d or defaults
         # set defaults
